from truegaze.plugins.adobe_mobile_sdk import AdobeMobileSdkPlugin
from truegaze.plugins.firebase import FirebasePlugin
from truegaze.plugins.weak_key import WeakKeyPlugin

# List of active plugins - when developing a new plugin, it should be added here.
# BasePlugin should never be added to this list.
ACTIVE_PLUGINS = [
    AdobeMobileSdkPlugin,
    FirebasePlugin,
    WeakKeyPlugin
]


#
# Copyright (c) 2019 Nightwatch Cybersecurity.
#
# This file is part of truegaze
# (see https://github.com/nightwatchcybersecurity/truegaze).
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
from androguard.core.bytecodes.apk import APK
import click
import requests
import tldextract

from truegaze.plugins.base import BasePlugin


# TODO: Add iOS support
# Plugin to check for insecure Firebase databases and GCP storage buckets
class FirebasePlugin(BasePlugin):
    name = 'FirebasePlugin'
    desc = 'Detection of insecure Firebase databases and GCP storage buckets'
    supports_android = True
    supports_ios = False
    supports_online = True

    # Main scanning method
    def scan(self):
        # Open the file
        apk = APK(self.filename)


        # Get the Firebase URL
        db_name = FirebasePlugin.get_db_name(apk)
        if db_name and len(db_name) > 0:
            click.echo('Found Firebase database: ' + db_name)
        else:
            click.echo('-- No Firebase database found, skipping...')
            return

        # If online check is disabled then skip
        if not self.is_online_testing_supported():
            click.echo('-- Online tests are disabled, skipping check...')
            return

        # Check if the database and bucket are accessible
        click.echo('-- Checking if the database is accessible...')
        messages = list()
        messages.append(FirebasePlugin.check_firebase_db(db_name))
        messages.append(FirebasePlugin.check_bucket(db_name))
        messages = list(filter(None, messages))

        # Show results if needed
        if len(messages) > 0:
            click.echo("-- Found " + str(len(messages)) + ' issues')
            for message in messages:
                click.echo(message)
        else:
            click.echo("-- No issues found")

    # Get the firebase URL from the APK
    # TODO: add tests
    @staticmethod
    def get_db_name(apk):
        res = apk.get_android_resources().get_string(apk.package, 'firebase_database_url')
        if res is not None:
            url = res[1]
            return tldextract.extract(url).subdomain

        return None

    # Check if the Firebase database is accessible
    @staticmethod
    def check_firebase_db(db_name):
        url = 'https://' + db_name + '.firebaseio.com/.json'
        res = requests.get(url, stream=True)
        if res.status_code == 200:
            return '---- ISSUE: Unprotected Firebase DB found - ' + url

        return None

    # Check if the bucket is accessible
    @staticmethod
    def check_bucket(db_name):
        url = 'https://storage.googleapis.com/' + db_name + '.appspot.com'
        res = requests.head(url)
        if res.status_code == 200:
            return '---- ISSUE: Unprotected bucket found - ' + url

        return None


#
# Copyright (c) 2019 Nightwatch Cybersecurity.
#
# This file is part of truegaze
# (see https://github.com/nightwatchcybersecurity/truegaze).
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

#
# Base plugin, implements basic methods and attributes all plugins must use. To create a new plugin,
# please extend this class, set the attributes correctly. implement the scan method and add the class
# to the ACTIVE_PLUGINS list in the truegaze.py file.
#


class BasePlugin(object):
    name = 'Base Plugin'
    desc = 'Base class used for all other plugins, do not access directly'

    # Whether scanning of Android files is supported
    supports_android = False

    # Whether scanning of iOS files is supported
    supports_ios = False

    # Whether supports online tests
    supports_online = False

    def __init__(self, filename, is_android, is_ios, do_online):
        # Main constructor
        #
        # :param filename: Filename to scan
        # :param is_android: Whether the provided file is an Android application
        # :param is_ios: Whether the provided is an iOS application
        # :param do_online: Whether online tests should be performed
        #
        self.filename = filename
        self.is_android = is_android
        self.is_ios = is_ios
        self.do_online = do_online

    # Utility method used to check if this plugin supports a given OS
    def is_os_supported(self):
        if (self.is_android and self.supports_android) or (self.is_ios and self.supports_ios):
            return True

        return False

    # Utility method used to check if this plugin supports online tests
    def is_online_testing_supported(self):
        if self.do_online and self.supports_online:
            return True

        return False

    # Main scanning method
    def scan(self):
        raise NotImplementedError('Scanning functionality not implemented')
